package com.xhkjedu.service;

import com.alibaba.fastjson2.JSON;
import com.xhkjedu.mapper.EPaperVerifyMapper;
import com.xhkjedu.model.EPaperQtypeQuestion;
import com.xhkjedu.model.EPaperStudent;
import com.xhkjedu.model.EPaperStudent0608;
import com.xhkjedu.model.EPaperStudentQuestion;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.PaperStudentParams;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @classname EPaperCorrectVerifyService
 * @description 考卷校验
 * @date 2024/6/8 14:39
 **/
@Service
public class EPaperVerifyService {
    @Resource
    private EPaperVerifyMapper ePaperVerifyMapper;

    //根据姓名搜索学生信息
    public List<Map> listStudentByStudentname(Integer examid, String studentname, Integer classid) {
        return ePaperVerifyMapper.listStudentByStudentname(examid, studentname, classid);
    }

    //获取学生试卷及试题作答
    public Map getStudentPaper(EPaperStudent0608 ps) {
        return ePaperVerifyMapper.getStudentPaper(ps);
    }

    //识别跳过
    public void correctSkip(EPaperStudent0608 ps) {
        ePaperVerifyMapper.correctSkip(ps);
    }

    //识别确认
    @Transactional(rollbackFor = Exception.class)
    public void correctSave(PaperStudentParams psParams){
        Integer epid = psParams.getEpid();
        Integer studentid = psParams.getStudentid();
        Integer newstudentid = psParams.getNewstudentid();

        Integer samed = 2; //是否和以前一致1一致2不一致
        if(!N_Utils.isTrueInteger(newstudentid) || studentid.equals(newstudentid)){
            samed = 1;
            newstudentid = studentid;
        }

        //先判断要确定的学生的试卷审核状态
        EPaperStudent psNew = ePaperVerifyMapper.getStuPaperInfoNew(epid,newstudentid);
        if(psNew.getVerified() == 1){
            //说明已经审核过，又出现，把要进来的试卷标记为审核不通过
            EPaperStudent0608 psV0608 = new EPaperStudent0608();
            psV0608.setVerified(2);
            psV0608.setEpid(epid);
            psV0608.setStudentid(studentid);
            psV0608.setSamed(0);
            ePaperVerifyMapper.updateStuPaper0608(psV0608);

            return;
        }

        //获取原试卷答案，
        List<EPaperQtypeQuestion> questions = ePaperVerifyMapper.listPaperAnswer(psParams.getEpid());

        List<EPaperStudentQuestion> stuQues = psParams.getQuestions();//仅包含客观题

        List<EPaperStudentQuestion> saveQues = new ArrayList<>();

        Double paperscore = 0D;//学生试卷得分

        for(EPaperStudentQuestion psq : stuQues){
            EPaperQtypeQuestion pqtypeQues = questions.stream().filter(a -> a.getEptqid().equals(psq.getEptqid())).findFirst().orElse(null);
            if(!N_Utils.isTrueInteger(psq.getAnswertype())){
                psq.setAnswertype(1);
            }
            setCommitQuestion(psq,pqtypeQues);

            paperscore = N_Utils.add(paperscore, psq.getStuscore());
            psq.setStudentid(newstudentid);
            psq.setEpid(epid);
            saveQues.add(psq);
        }

        //获取试卷中主观题
        List<EPaperStudentQuestion> stuQuesZgt = ePaperVerifyMapper.listQuestionZGT(epid,studentid);
        for(EPaperStudentQuestion psqZgt : stuQuesZgt){
            psqZgt.setStudentid(newstudentid);
            paperscore = N_Utils.add(paperscore, psqZgt.getStuscore());
            saveQues.add(psqZgt);
        }

        ePaperVerifyMapper.updateBatchStuQuestion(saveQues);

        //获取原试卷信息
        EPaperStudent ps = ePaperVerifyMapper.getStuPaperInfo(epid,studentid);

        //更改新学生信息
        EPaperStudent paperStudent = new EPaperStudent();
        paperStudent.setEpid(epid);
        paperStudent.setStudentid(newstudentid);
        paperStudent.setStuscore(paperscore);
        paperStudent.setStupic(ps.getStupic());
        paperStudent.setPagenum(ps.getPagenum());
        paperStudent.setBatchid(ps.getBatchid());
        paperStudent.setSstate(2);
        paperStudent.setChecked(2);
        paperStudent.setVerified(1);
        ePaperVerifyMapper.updateStuPaper(paperStudent);

        //新表中更改状态
        EPaperStudent0608 ps0608 = new EPaperStudent0608();
        ps0608.setEpid(epid);
        ps0608.setStudentid(studentid);
        ps0608.setVerified(1);
        ps0608.setSamed(samed);
        ePaperVerifyMapper.updateStuPaper0608(ps0608);

        if(samed == 2){
            //说明学生信息布一致，清空表中原学生试卷及试题信息
            ePaperVerifyMapper.clearPaperStuQues(epid,studentid);
            ePaperVerifyMapper.clearPaperStu(epid,studentid);
        }
    }

    //判断单道题情况
    private void setCommitQuestion(EPaperStudentQuestion psq, EPaperQtypeQuestion answerVo) {
        int timestamp = N_Utils.getSecondTimestamp();
        if(!N_Utils.isTrueInteger(psq.getAnswertime())){
            psq.setAnswertime(timestamp);
        }

        String stuanswer = psq.getStuanswer();
        String stuanswertxt = psq.getStuanswertxt();
        Double stuscore = 0D;
        if (N_Utils.isEmpty(stuanswer) && N_Utils.isEmpty(stuanswertxt)
                && N_Utils.isListEmpty(JSON.parseArray(stuanswer, String.class))
                && N_Utils.isListEmpty(JSON.parseArray(stuanswertxt, String.class))) {
            psq.setAnswered(0);
            psq.setStuanswer(null);
            psq.setStuanswertxt(null);
            psq.setStuscore(stuscore);
            psq.setChecked(2);
            psq.setChecktime(timestamp);
            psq.setAnswered(0);
        } else {
            List<String> stuanswers = JSON.parseArray(stuanswer, String.class);
            if(stuanswers.get(0).equals("null") || "".equals(stuanswers.get(0))){
                //说明是空
                psq.setAnswered(0);
                psq.setStuanswer(null);
                psq.setStuanswertxt(null);
                psq.setStuscore(stuscore);
                psq.setChecked(2);
                psq.setChecktime(timestamp);
                psq.setAnswered(0);
            }else{
                String qanswer = answerVo.getAnswer();
                List<String> qanswers = JSON.parseArray(qanswer, String.class);
                int ctype = answerVo.getCtype();
                if (psq.getAnswertype() == 1) {
                    if (N_Utils.isObjectiveQuestion(ctype)) {
                        //客观题进行批阅
                        if (ctype == 1 || ctype == 4 || ctype == 5 || ctype == 6) {
                            //单选、判断
                            if (stuanswer.equalsIgnoreCase(qanswer)) {
                                stuscore = answerVo.getScore();
                            }
                        } else if (ctype == 2) {//多选
                            //多选、选错不得分
                            if (stuanswers.get(0).length() <= qanswer.length()) {
                                String[] qanswerList = qanswers.get(0).split("");
                                String[] stuanswerList = stuanswers.get(0).split("");
                                int rightNum = 0;//选对个数
                                for (String sa : stuanswerList) {
                                    int right = 0;
                                    for (String qa : qanswerList) {
                                        if (qa.equals(sa)) {
                                            right = 1;
                                            rightNum++;
                                        }
                                    }
                                    if (right == 0) {//说明选错
                                        rightNum = 0;
                                        break;
                                    }
                                }
                                if (rightNum > 0) {
                                    if (qanswerList.length == rightNum) {
                                        //全选对
                                        stuscore = answerVo.getScore();
                                    } else if (qanswerList.length > rightNum) {
                                        //少选得半分
                                        stuscore = N_Utils.div(answerVo.getScore(), 2, 1);
                                    }
                                }
                            }
                        }
                        psq.setChecked(2);
                        psq.setChecktime(timestamp);
                    } else {
                        psq.setChecked(0);
                    }
                } else {
                    if(N_Utils.isNotEmpty(stuanswertxt)){
                        List<String> stuanswertxt_list = JSON.parseArray(stuanswertxt,String.class);
                        String stuanswertxt_txt = stuanswertxt_list.get(0);
                        if(N_Utils.isNotEmpty(stuanswertxt_txt)){
                            stuanswertxt_txt = N_Utils.strTrim(stuanswertxt_txt);
                            if(N_Utils.isNotEmpty(stuanswertxt_txt)){
                                psq.setChecked(0);
                                List<String> stuanswertxt_list_new = new ArrayList<>();
                                stuanswertxt_list_new.add(stuanswertxt_txt);
                                psq.setStuanswertxt(JSON.toJSONString(stuanswertxt_list_new));
                            }else{
                                psq.setChecked(2);
                                psq.setStuanswertxt("[\"\"]");
                            }
                        }else{
                            psq.setChecked(2);
                            psq.setStuanswertxt("[\"\"]");
                        }
                    }else{
                        psq.setChecked(2);
                    }
                }

                String newStuAnswer = stuanswers.get(0).toUpperCase();
                List<String> newStuAnswers = new ArrayList<>();
                newStuAnswers.add(newStuAnswer);
                psq.setStuanswer(JSON.toJSONString(newStuAnswers));

                psq.setStuscore(stuscore);
                psq.setAnswered(1);
            }
        }
    }


}

