package com.xhkjedu.utils;

import com.xhkjedu.exception.MissingParametersException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * @author WN
 * @className N_Utils
 * @description 工具类
 * @date 2019/4/3 14:13
 **/
@Slf4j
public class N_Utils {

    /**
     * 字符串是否为空
     *
     * @param: @param obj
     * @param: @return
     * @author WN
     */
    public static boolean isNotEmpty(Object obj) {
        if (null == obj || "".equals(obj.toString().trim()) || "null".equals(obj.toString().trim())) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * 字符串是否为空
     *
     * @param: @param obj
     * @param: @return
     * @author zj
     */
    public static boolean isEmpty(Object obj) {
        if (null == obj || "".equals(obj.toString().trim()) || "null".equals(obj.toString().trim())) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * json字符串是否不为空
     * @Param [jsonStr]
     * @Author ywx
     * @Date 2020/5/13 16:06
     * @return boolean
     **/
    public static boolean isJsonStrNotEmpty(String jsonStr) {
        if (isNotEmpty(jsonStr) && !jsonStr.equals("[]")){
            return true;
        } else {
            return false;
        }
    }


    /**
     * 功能描述  integer 是否是有效的类型
     *
     * @param * @param obj
     * @return boolean
     * @author WN
     * @date 2019/7/15
     */
    public static boolean isTrueInteger(Integer obj) {
        if (null != obj && obj > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 获取当前时间精确到秒的时间戳
     *
     * @param @return 设定文件
     * @return int    返回类型
     * @Title: getSecondTimestamp
     * @author WN
     * @date 2019年3月6日 下午10:13:50
     */
    public static int getSecondTimestamp(){
        String timestamp = String.valueOf(SystemClock.millisClock().now()/1000);
        return Integer.valueOf(timestamp);
    }

    public static String getStrtimeTimestamp(Integer seconds,String format){
        if(seconds == null || seconds==0){
            return "";
        }
        if(format == null || format.isEmpty()){
            format = "yyyy-MM-dd HH:mm:ss";
        }
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(new Date(Long.valueOf(seconds+"000")));
    }

    //计算两个时间戳相差的天数
    public static Integer getTimestampDay(Integer begintime,Integer endtime){
        if(isEmptyInteger(begintime)) return 365;//默认时间差为一年
        /*Integer day = 0;

        Integer cha = endtime - begintime;

        day = cha / (3600 * 24);*/

        BigDecimal b1 = new BigDecimal(String.valueOf(endtime - begintime));
        BigDecimal b2 = new BigDecimal(String.valueOf(3600 * 24));

        BigDecimal count = b1.divide(b2,2,BigDecimal.ROUND_UP);
        return count.intValue();
    }

    /**
     * 字符串转数字
     * @Param [str]
     * @Author ywx
     * @Date 2020/6/3 17:01
     * @return java.lang.Integer
     **/
    public static Integer str2Int(String str){
        if (isNotEmpty(str)){
            return Integer.parseInt(str);
        } else {
            return 0;
        }
    }

    /**
     * 日期转时间戳(秒)
     * @Param [data, strType]
     * @Author ywx
     * @Date 2020/6/4 17:47
     * @return int
     **/
    public static int getDataToTimestamp(String data,String strType) {
        try {
            SimpleDateFormat format =  new SimpleDateFormat(strType);
            Date date = format.parse(data);
            //日期转时间戳（毫秒）
            return Integer.valueOf((date.getTime()/1000)+"");
        } catch (ParseException e) {
            log.error("日期转时间戳失败",e.getMessage());
            return 0;
        }
    }

    /**
     * 判断集合中数据同一个位置是否相同
     *
     * @param @param  oldlst
     * @param @param  newlst
     * @param @return 设定文件
     * @return boolean    返回类型
     * @Title: listNeworder
     * @author WN
     * @date 2018年8月2日 下午6:02:59
     */
    public static boolean listNeworder(List<Integer> oldlst, List<Integer> newlst) {
        for (int i = 0; i < oldlst.size(); i++) {
            int oldid = oldlst.get(i);
            int newid = newlst.get(i);
            if (oldid == newid) {
                return true;
            }
        }
        return false;
    }

    /**
     * 将一个list均分成n个list,主要通过偏移量来实现的
     * @param source
     * @return
     */
    public static List<List<Integer>> averageAssignList(List<Integer> source,int n){
        List<List<Integer>> result=new ArrayList<List<Integer>>();
        int remaider=source.size()%n;  //(先计算出余数)
        int number=source.size()/n;  //然后是商
        int offset=0;//偏移量
        for(int i=0;i<n;i++){
            List<Integer> value=null;
            if(remaider>0){
                value=source.subList(i*number+offset, (i+1)*number+offset+1);
                remaider--;
                offset++;
            }else{
                value=source.subList(i*number+offset, (i+1)*number+offset);
            }
            result.add(value);
        }
        return result;
    }

    /**
     * 判断集合是否不为空
     * @Param [list]
     * @Author ywx
     * @Date 2020/6/15 10:50
     * @return boolean
     **/
    public static boolean isListNotEmpty(List list) {
        if (CollectionUtils.isEmpty(list)){
            return false;
        } else {
            return true;
        }
    }

    /**
     * 判断集合是否为空
     * @Param [list]
     * @Author ywx
     * @Date 2020/7/14 8:48
     * @return boolean
     **/
    public static boolean isListEmpty(List list) {
        if (CollectionUtils.isEmpty(list)){
            return true;
        } else {
            return false;
        }
    }

    //获取当前日期的节点时间（年，月，日，时，分，秒）
    public static Integer getNodeDateTime(String node) {
        LocalDateTime today = LocalDateTime.now();
        Integer resultNode = null;
        switch (node) {
            case "year":
                resultNode = today.getYear();
                break;
            case "month":
                resultNode = today.getMonthValue();
                break;
            case "day":
                resultNode = today.getDayOfMonth();
                break;
            case "hour":
                resultNode = today.getHour();
                break;
            case "minute":
                resultNode = today.getMinute();
                break;
            case "second":
                resultNode = today.getSecond();
                break;
            default:
                // 当前日期是当前年的第几天。例如：2019/1/3是2019年的第三天
                resultNode = today.getDayOfYear();
                break;
        }
        return resultNode;
    }

    /**
     * 数字转字母 1-26 ： A-Z
     * @param num
     * @return
     */
    public static String numberToLetter(int num) {
        if (num <= 0) {
            return null;
        }
        String letter = "";
        num--;
        do {
            if (letter.length() > 0) {
                num--;
            }
            letter = ((char) (num % 26 + (int) 'A')) + letter;
            num = (int) ((num - num % 26) / 26);
        } while (num > 0);

        return letter;
    }

    /**
     * 获取指定个数的英文字母数组
     * @Param [num]
     * @Author ywx
     * @Date 2019/11/25 17:06
     * @return java.lang.String[]
     **/
    public static String[] getLetter(Integer num){
        String[] sections = new String[num];
        if(num!=null && num>0){
            for(int i=1;i<=num;i++){
                sections[i-1] = numberToLetter(i);
            }
        }
        return sections;
    }

    /**
     * 集合1不包含在集合2中的数字集合
     * @Param [nums1, nums2]
     * @Author ywx
     * @Date 2020/7/8 13:01
     * @return java.util.List<java.lang.Integer>
     **/
    public static String listNotInList2(List<String> nums1,List<String> nums2){
        List<String> nums = new ArrayList<>();
        for (String num : nums1) {
            if (!nums2.contains(num)){
                nums.add(num);
            }
        }

        return String.join(",",nums);
    }

    /**
     * 集合1不包含在集合2中的数字集合
     * @Param [nums1, nums2]
     * @Author ywx
     * @Date 2021/3/24 15:42
     * @return java.util.List<java.lang.String>
     **/
    public static List<String> listNotInList3(List<String> nums1,List<String> nums2){
        List<String> nums = new ArrayList<>();
        for (String num : nums1) {
            if (!nums2.contains(num)){
                nums.add(num);
            }
        }
        return nums;
    }

    /**
     * 数字是否包含在集合中的数字集合
     * @Param [num, nums]
     * @Author ywx
     * @Date 2020/7/14 11:18
     * @return boolean
     **/
    public static boolean numInList(Integer num,List<Integer> nums){
        if (isListEmpty(nums)) return false;
        if (nums.contains(new Integer(num))){
            return true;
        } else {
            return false;
        }
    }

    /**
     * 集合去重
     * @Param [list]
     * @Author ywx
     * @Date 2020/7/11 9:59
     * @return java.util.List
     **/
    public static List listDistinct(List<?> list){
        return list.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 功能描述  判断integer类型不为null和0
     *
     * @param * @param strint
     * @return boolean
     * @author WN
     * @date 2019/10/15
     */
    public static boolean isEmptyInteger(Integer strint) {
        if (null != strint && strint != 0) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * 去除字符串中前后的全、半角空格
     * @Param [str]
     * @Author ywx
     * @Date 2020/4/1 9:57
     * @return java.lang.String
     **/
    public static String strTrim(String str) {
        str = str.trim();//去除字符串前后的半角空格
        while (str.startsWith("　")) {//去除字符串前的全角空格
            str = str.substring(1, str.length()).trim();
        }
        while (str.endsWith("　")) {//去除字符串后的全角空格
            str = str.substring(0, str.length() - 1).trim();
        }
        return str;
    }



    /**
     * 验证手机号
     * @param phone
     * @return
     */
    public static boolean isPhone(String phone) {
        //String regex = "^((13[0-9])|(14[5,6,7,9])|(15([0-3]|[5-9]))|(166)|(17[0,1,3,5,6,7,8])|(18[0-9])|(19[8|9]))\\d{8}$";
        String regex = "^1\\d{10}$";
        if (phone == null) {
            return false;
        } else {
            Pattern p = Pattern.compile(regex);
            Matcher m = p.matcher(phone);
            boolean isMatch = m.matches();
            return isMatch;
        }

    }

    /**
     * 参数校验，为空的话提示定义的提示信息
     * @Param [strs] strs={"filename","msg","type","filename2","msg2","type2"}
     * @Author ywx
     * @Date 2020/4/26 15:08
     * @return void
     **/
    public static void validation(Object[] strs) {
        for (int i = 0; i < strs.length; i+=3) {
            Object filename = strs[i];
            String msg = strs[i+1].toString()+"不能为空";
            Integer type = Integer.parseInt(strs[i+2].toString());
            if(type == 1){//Integer类型参数
                if (!N_Utils.isNotEmpty(filename) || filename.equals(0)) {
                    throw new MissingParametersException(msg);
                }
            } else {//String类型参数
                if (!N_Utils.isNotEmpty(filename)) {
                    throw new MissingParametersException(msg);
                }
            }
        }
    }


    /**
     *功能描述  获取当前年月日的时间戳
     * @author WN
     * @date 2020/8/9
     * @param  * @param
     * @return java.lang.Long
     */
    public static Long getNowtimestamp(){
        LocalDate localDate = LocalDate.now();

        ZoneId zone = ZoneId.systemDefault();
        LocalDateTime ldt = localDate.atTime(23,59,59);
        Instant instant = ldt.atZone(zone).toInstant();
        Long stamp =instant.toEpochMilli()/1000;
        return stamp;
    }

    public static Long getNowtimestamp2(){
        LocalDate localDate = LocalDate.now();

        ZoneId zone = ZoneId.systemDefault();
        LocalDateTime ldt = localDate.atTime(00,00,00);
        Instant instant = ldt.atZone(zone).toInstant();
        Long stamp =instant.toEpochMilli()/1000;
        return stamp;
    }

    /**
     *功能描述  根据当前时间获取指定日期的时间戳
     * @author WN
     * @date 2020/8/9
     * @param  * @param type
     * @return java.lang.String
     */
    public static Long getLongGoTime(Integer type){
        LocalDate localDate = LocalDate.now();
        ZoneId zone = ZoneId.systemDefault();
        Long stamp = null;

        LocalDate godate = null;

        if(type==1){
            //过去一周
            godate = localDate.minusWeeks(1);
        }else if(type==2){
            //过去一月
            godate = localDate.minusMonths(1);
        }else if(type==3){
            //过去三个月
            godate = localDate.minusMonths(3);
        }else if(type==4){
            //过去六个月
            godate = localDate.minusMonths(6);

        }else{
            //过去一年
            godate = localDate.minusYears(1);
        }
        stamp = godate.atStartOfDay(zone).toInstant().toEpochMilli()/1000;


        return stamp;
    }

    /**
     * @author ywx
     * @description 去除a标签
     * @date 2020/8/26 20:30
     **/
    public static String delATag(String str){
        if (isEmpty(str)){
            return null;
        } else {
            return str.replaceAll("<a[^>]*>|</a>","");
        }
    }


    /**
     *功能描述 两个数相除并乘以100.保留2位小数
     * @author WN
     * @date 2020/8/8
     * @param  * @param linecount
     * @param studentnum
     * @return java.math.BigDecimal
     */
    public static double getIntegerDivideAndMulitiply(Integer linecount, Integer studentnum){
        if (isEmptyInteger(studentnum) || isEmptyInteger(linecount)) {
            return 0;
        }
        BigDecimal b1 = new BigDecimal(linecount.toString());
        BigDecimal b2 = new BigDecimal(studentnum.toString());
        BigDecimal b = new BigDecimal(100);

        BigDecimal count = b1.divide(b2,4,BigDecimal.ROUND_DOWN).multiply(b);
        return count.doubleValue();
    }

    //两个数相除并乘以100.保留2位小数
    public static double getDoubleDivideAndMulitiply(Double linecount, Double studentnum){
        if (studentnum.compareTo(new Double(0)) == 0 || linecount == null) {
            return 0;
        }
        BigDecimal b1 = new BigDecimal(linecount.toString());
        BigDecimal b2 = new BigDecimal(studentnum.toString());
        BigDecimal b = new BigDecimal(100);

        BigDecimal count = b1.divide(b2,4,BigDecimal.ROUND_DOWN).multiply(b);
        return count.doubleValue();
    }

    //两个数相除并乘以100.保留2位小数
    public static double getDoubleDivideAndMulitiply(Double linecount, Integer studentnum){
        if (isEmptyInteger(studentnum) || linecount == null) {
            return 0;
        }
        BigDecimal b1 = new BigDecimal(linecount.toString());
        BigDecimal b2 = new BigDecimal(studentnum.toString());
        BigDecimal b = new BigDecimal(100);

        BigDecimal count = b1.divide(b2,4,BigDecimal.ROUND_DOWN).multiply(b);
        return count.doubleValue();
    }

    //两个数相除，保留两位小数（平均分）
    public static double getDoubleDivideForAvg(Double linecount, Integer studentnum){
        if (isEmptyInteger(studentnum) || linecount == null) {
            return 0;
        }
        BigDecimal b1 = new BigDecimal(linecount.toString());
        BigDecimal b2 = new BigDecimal(studentnum.toString());

        BigDecimal count = b1.divide(b2,2,BigDecimal.ROUND_DOWN);
        return count.doubleValue();
    }

    //两个数相除，保留两位小数（平均分）
    public static double getDoubleDivideForAvg(Integer linecount, Integer studentnum){
        if (isEmptyInteger(studentnum) || linecount == null) {
            return 0;
        }
        BigDecimal b1 = new BigDecimal(linecount.toString());
        BigDecimal b2 = new BigDecimal(studentnum.toString());

        BigDecimal count = b1.divide(b2,2,BigDecimal.ROUND_DOWN);
        return count.doubleValue();
    }

    //两个double 类型相加
    public static Double getDoubleSum(Double value1,Double value2){
        BigDecimal b1 = new BigDecimal(Double.toString(value1));
        BigDecimal b2 = new BigDecimal(Double.toString(value2));

        return b1.add(b2).doubleValue();
    }
    //两个double 类型相减
    public static Double getDoubleReduce(Double value1,Double value2){
        BigDecimal b1 = new BigDecimal(Double.toString(value1));
        BigDecimal b2 = new BigDecimal(Double.toString(value2));

        return b1.subtract(b2).doubleValue();
    }

    //计算比例向下取整
    public static Integer floorDiv(Integer num,Integer totalNum) {
        if (isEmptyInteger(totalNum) || isEmptyInteger(num)) return 0;
        return Math.floorDiv(num * 100, totalNum);
    }

    /**
     * 格式化百分比
     * @Param [num, totalNum]
     * @Author ywx
     * @Date 2020/9/22 10:26
     * @return java.lang.Integer
     **/
    public static Integer formatProportion(Integer num,Integer totalNum){
        if (isEmptyInteger(totalNum) || isEmptyInteger(num)) return 0;
        BigDecimal countBigDecimal = new BigDecimal(num);
        BigDecimal totalCountBigDecimal = new BigDecimal(totalNum);
        BigDecimal divide = countBigDecimal.divide(totalCountBigDecimal, 20, BigDecimal.ROUND_HALF_UP);
        divide=divide.setScale(2,BigDecimal.ROUND_HALF_UP);
        return divide.multiply(new BigDecimal("100")).intValue();
    }

    public static Integer formatAvg(Integer num,Integer totalNum){
        if (isEmptyInteger(totalNum) || isEmptyInteger(num)) return 0;
        BigDecimal countBigDecimal = new BigDecimal(num);
        BigDecimal totalCountBigDecimal = new BigDecimal(totalNum);
        BigDecimal divide = countBigDecimal.divide(totalCountBigDecimal, 0, BigDecimal.ROUND_UP);
        return divide.intValue();
    }

    /**
     * 格式化数值
     * @Param [num数值, decimalNum保留小数位数]
     * @Author ywx
     * @Date 2021/5/28 9:57
     * @return java.lang.Double
     **/
    public static Double formatDouble(Double num,Integer decimalNum) {
        if (num == null || num == Double.NEGATIVE_INFINITY || num == Double.POSITIVE_INFINITY) return 0.0;
        if (isEmptyInteger(decimalNum)) return num;
        BigDecimal bg = new BigDecimal(num);
        double num2 = bg.setScale(decimalNum, BigDecimal.ROUND_DOWN).doubleValue();
        return num2;
    }

    /**
     * @Description DoubleSummaryStatistics获取最大值有可能为(-Infinity)、最小值有可能为(Infinity);
     * 检查Double类型的值是否为-Infinity或Infinity，并在是的情况下返回0
     * @Date 2023/12/28 16:30
     * @Author YWX
     * @Param [value]
     * @Return double
     **/
    public static double handleInfinity(double value) {
        if (value == Double.NEGATIVE_INFINITY || value == Double.POSITIVE_INFINITY) {
            return 0;
        } else {
            return value;
        }
    }

    /**
     * @Description  格式化double类型，如果未小数点后为0反正整式，
     * @Param [score]
     * @Return java.lang.String
     * @Author wn
     * @Date 2022/9/2 16:34
     **/
    public static String formatDouble(Double score){
        BigDecimal bg = new BigDecimal(score).setScale(1, RoundingMode.HALF_DOWN);
        double num = bg.doubleValue();
        if (Math.round(num) - num == 0) {
            return String.valueOf((long) num);
        }
        return String.valueOf(num);
    }

    /**
     * @Description 格式化数值集合百分比（总和为100）
     * @Date 2023/10/25 17:36
     * @Author YWX
     * @Param [nums]
     * @Return java.util.Map<java.lang.Integer,java.lang.Double>
     **/
    public static Map<Integer, Double> parsePercent(List<Integer> nums) {
        Map<Integer, Double> map = new HashMap<>();
        int sum = 0;//定义一个整数变量用于记录列表中所有数字的总和
        for (int num : nums) {
            sum += num;
        }

        DecimalFormat df = new DecimalFormat("#.##"); // 用于格式化百分比，保留两位小数
        for (int num : nums) {
            double percentage = (double) num / sum * 100;//计算数字在总和中的百分比
            map.put(num, Double.valueOf(df.format(percentage)));//将数字作为键，格式化后的百分比作为值，存入哈希映射中
        }
        return map;//返回包含数字及其对应百分比的哈希映射
    }

    static String CHN_NUMBER[] = { "零", "一", "二", "三", "四", "五", "六", "七", "八", "九" };
    static String CHN_UNIT[] = { "", "十", "百", "千" }; // 权位
    static String CHN_UNIT_SECTION[] = { "", "万", "亿", "万亿" }; // 节权位

    /**
     * 将四位的section转换为中文数字
     *
     * @param section
     * @return
     */
    public static String SectionNumToChn(int section) {
        StringBuffer returnStr = new StringBuffer();
        int unitPos = 0; // 节权位的位置编号，0-3依次为个十百千;

        Boolean zero = true;
        while (section > 0) {

            int v = (section % 10);
            if (v == 0) {
                if ((section == 0) || !zero) {
                    zero = true; /* 需要补0，zero的作用是确保对连续的多个0，只补一个中文零 */
                    // chnStr.insert(0, chnNumChar[v]);
                    returnStr.insert(0, CHN_NUMBER[v]);
                }
            } else {
                zero = false; // 至少有一个数字不是0
                StringBuffer tempStr = new StringBuffer(CHN_NUMBER[v]);// 数字v所对应的中文数字
                tempStr.append(CHN_UNIT[unitPos]); // 数字v所对应的中文权位
                returnStr.insert(0, tempStr);
            }
            unitPos++; // 移位
            section = section / 10;
        }
        return returnStr.toString();
    }


    static int f(int a,int b){  // 求a和b的最大公约数
        if(a < b){
            int c = a;
            a = b;
            b = c;
        }
        int r = a % b;
        while(r != 0){
            a = b;
            b = r;
            r = a % b;
        }
        return b;
    }



    //处理题型中客观题1选择题2多选题456判断7完型填空全选择（弃用）8阅读理解全选择（弃用）10任务行阅读券选择（弃用）16完型填空复合17任务型阅读复合
    private static int[] octypelist = new int[]{1,2,4,5,6,7,8,10,16,17};
    /**
     * @Description 判断ctype是否是客观题
     * @Param [ctype]
     * @Return boolean 如果是客观题返回true,否则返回false
     * @Author wn
     * @Date 2022/7/20 15:20
     **/
    public static boolean isObjectiveQuestion(Integer ctype){
        if(ctype != null && ctype != 0){
            return IntStream.of(octypelist).anyMatch(x -> x == ctype);
        }else{
            return false;
        }
    }
    //简单题进行分项展示
    private static int[] simplelist = new int[]{1,2,4,5,6,7,8,10,16,17};
    public static boolean isSimpleCheckQuestion(Integer ctype){
        if(ctype != null && ctype != 0){
            return IntStream.of(simplelist).anyMatch(x -> x == ctype);
        }else{
            return false;
        }
    }


    /**
     * 提供（相对）精确的除法运算。当发生除不尽的情况时，由scale参数指 定精度，以后的数字舍弃。
     *
     * @param v1    被除数
     * @param v2    除数
     * @param scale 表示表示需要精确到小数点以后几位。
     * @return 两个参数的商
     */
    public static double div(double v1, double v2, int scale) {
        try {
            if (scale < 0) {
                throw new IllegalArgumentException("The   scale   must   be   a   positive   integer   or   zero");
            }
            BigDecimal b1 = new BigDecimal(Double.toString(v1));
            BigDecimal b2 = new BigDecimal(Double.toString(v2));
            return b1.divide(b2, scale, BigDecimal.ROUND_DOWN).doubleValue();
        } catch (Exception e) {
            return 0;
        }
    }

    /**
     * 提供精确的加法运算。
     *
     * @param v1 被加数
     * @param v2 加数
     * @return 两个参数的和
     */
    public static double add(double v1, double v2) {
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.add(b2).doubleValue();
    }
}
